# lib-interceptor
Library for executing reflected [ClassData](https://github.com/MRazvan/lib-reflect#ClassData) and allowing intercepting of the execution or result.

It is using the [lib-reflect](https://github.com/MRazvan/lib-reflect) library in order to generate the needed information for executing a method on a class.

The execution will happen in an execution context. The execution of the method(s) does not throw even if an error happens. The error will be available on the context.


```typescript
// Interceptors for method execution
class Log implements IBeforeActivation {
  public before(ctx: IContext): boolean {
    console.log('BEFORE: ', ctx.getActivation());
    // Let the execution continue after this method
    return true;
  }
}

class Performance implements IBeforeActivation, IAfterActivation {
  public before(ctx: IContext): boolean {
    console.log('Start Performance');
    ctx.setData('Start', process.hrtime());
    // Let the execution continue after this method
    return true;
  }
  public after(ctx: IContext): void {
    const start = ctx.getData<[number, number]>('Start');
    const time = process.hrtime(start);
    console.log(`Took : ${time[0] * 1000 + time[1] / 1e6} ms`);
  }
}

// Create a test class
class TestClass {
  // Decorate with the logger
  @UseBefore(Log)
  @UseActivation(Performance)
  public myMethod(): string {
    return 'From my handler';
  }
}

const generator = new ActivationsGenerator();
generator.register(TestClass);
// Create the inversify container
const container = new Container();
// Generate the activations
// We have only one class with one method
const activation = generator.generateActivations(container)[0];
// Create the context of execution
const context = new DefaultContext(container, activation);
context.execute().then(() => {
  console.log('RESULT', context.getResult());
});
```